from ldnode.calculator.schemas import CB_BUILTIN_FACTORS


def test_cb_builtin_factors_include_new_source_columns() -> None:
    assert "adj_factor" in CB_BUILTIN_FACTORS
    assert "volume" in CB_BUILTIN_FACTORS
    assert "amt" in CB_BUILTIN_FACTORS
    assert "remaining_size" in CB_BUILTIN_FACTORS
    assert "issuance_size" in CB_BUILTIN_FACTORS
    assert "redeem_real_days" in CB_BUILTIN_FACTORS
    assert "redeem_count_days" in CB_BUILTIN_FACTORS
    assert "redeem_total_days" in CB_BUILTIN_FACTORS


def test_cb_builtin_factors_exclude_realtime_cashflow_source_columns() -> None:
    assert "pay_interest_day_list" not in CB_BUILTIN_FACTORS
