from ldnode.loader.files import CB_DATA, CB_PRE_QUOTATION


def test_cb_backtest_data_uses_convertible_bond_path() -> None:
    assert CB_DATA.key == "backtest_data/convertible_bond.parquet"


def test_cb_pre_quotation_uses_convertible_bond_path() -> None:
    assert CB_PRE_QUOTATION.key == "before_trading_data/convertible_bond.parquet"
